def get_path():
    raise NotImplementedError


def get_absolute_path():
    raise NotImplementedError


def get_canonical_path():
    raise NotImplementedError


def get_cwd_path():
    raise NotImplementedError


def is_absolute_path():
    raise NotImplementedError
