def is_blank():
    "字符串为 null 或者内部字符全部为 ' ' '\t' '\n' '\r' 这四类字符时返回 true"
    raise NotImplementedError


def not_blank():
    raise NotImplementedError


def snake_to_camel_case():
    raise NotImplementedError


def camel_to_snake_case():
    raise NotImplementedError


def capitalize(s: str):
    return s.capitalize()
